@@ -82,6 +82,12 @@ int crypto_keys_subr_generate_RSA(RSA **, RSA **);
  */
 int crypto_keys_subr_generate_HMAC(struct crypto_hmac_key **);
 
+/**
+ * crypto_keys_subr_free_HMAC(key):
+ * Free an HMAC key.
+ */
+void crypto_keys_subr_free_HMAC(struct crypto_hmac_key **);
+
 /**
  * crypto_file_init_keys(void):
  * Initialize the keys cached by crypto_file.
