@@ -3,10 +3,11 @@
 #include <errno.h>
 #include <stdint.h>
 #include <stdlib.h>
-#include <string.h>
 
+#include <openssl/crypto.h>
 #include <openssl/err.h>
 #include <openssl/rand.h>
+#include <openssl/rsa.h>
 
 #include "crypto_entropy.h"
 #include "sysendian.h"
@@ -161,12 +162,16 @@ crypto_keys_init(void)
 	RAND_seed(randbuf, RANDBUFLEN);
 
 	/* Load server root public key. */
-	if (crypto_keys_server_import_root())
+	if (crypto_keys_server_import_root()) {
+		warn0("Could not import server root public key");
 		goto err0;
+	}
 
 	/* Initialize keys owned by crypto_file. */
-	if (crypto_file_init_keys())
+	if (crypto_file_init_keys()) {
+		warn0("Could not initialize crypto_file keys");
 		goto err0;
+	}
 
 	/* Success! */
 	return (0);
@@ -472,7 +477,7 @@ crypto_keys_export(int keys, uint8_t ** buf, size_t * buflen)
 		/* Write key header. */
 		kh = *buf + bufpos;
 		le32enc(&kh[KEYHEADER_OFFSET_LEN], len);
-		kh[KEYHEADER_OFFSET_TYPE] = key;
+		kh[KEYHEADER_OFFSET_TYPE] = key & 0xff;
 
 		/* Advance buffer position. */
 		bufpos += KEYHEADER_LEN + len;
