@@ -1,15 +1,17 @@
 #include "bsdtar_platform.h"
 
+#include <assert.h>
 #include <errno.h>
+#include <limits.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include <openssl/bn.h>
 #include <openssl/err.h>
-#include <openssl/rand.h>
 #include <openssl/rsa.h>
 
+#include "crypto_compat.h"
 #include "crypto_entropy.h"
 #include "sysendian.h"
 #include "warnp.h"
@@ -31,7 +33,7 @@
  */
 
 static int import_BN(BIGNUM **, const uint8_t **, size_t *);
-static int export_BN(BIGNUM *, uint8_t **, size_t *, uint32_t *);
+static int export_BN(const BIGNUM *, uint8_t **, size_t *, uint32_t *);
 
 /**
  * import_BN(bn, buf, buflen):
@@ -54,6 +56,12 @@ import_BN(BIGNUM ** bn, const uint8_t ** buf, size_t * buflen)
 	*buf += sizeof(uint32_t);
 	*buflen -= sizeof(uint32_t);
 
+	/* Sanity check. */
+	if (len > INT_MAX) {
+		warn0("Unexpected key length");
+		goto err0;
+	}
+
 	/* Make sure there's enough data. */
 	if (*buflen < len) {
 		warn0("Unexpected EOF of key data");
@@ -68,7 +76,7 @@ import_BN(BIGNUM ** bn, const uint8_t ** buf, size_t * buflen)
 		goto err0;
 	for (i = 0; i < len; i++)
 		bnbuf[len - 1 - i] = (*buf)[i];
-	if ((*bn = BN_bin2bn(bnbuf, len, NULL)) == NULL) {
+	if ((*bn = BN_bin2bn(bnbuf, (int)len, NULL)) == NULL) {
 		warn0("%s", ERR_error_string(ERR_get_error(), NULL));
 		goto err1;
 	}
@@ -95,14 +103,14 @@ err0:
  * Add the required storage length to ${len}.
  */
 static int
-export_BN(BIGNUM * bn, uint8_t ** buf, size_t * buflen,
+export_BN(const BIGNUM * bn, uint8_t ** buf, size_t * buflen,
     uint32_t * len)
 {
 	size_t i;
 	unsigned int bnlen;
 
 	/* Figure out how much space we need. */
-	bnlen = BN_num_bytes(bn);
+	bnlen = (unsigned int)BN_num_bytes(bn);
 
 	/* Add the required storage length to ${len}. */
 	if (*len + sizeof(uint32_t) < *len) {
@@ -163,10 +171,15 @@ err0:
 int
 crypto_keys_subr_import_RSA_priv(RSA ** key, const uint8_t * buf, size_t buflen)
 {
+	BIGNUM * n, * e, * d, * p, * q, * dmp1, * dmq1, * iqmp;
+
+	/* This simplifies the error path cleanup. */
+	n = e = d = p = q = dmp1 = dmq1 = iqmp = NULL;
 
 	/* Free any existing key. */
 	if (*key != NULL)
 		RSA_free(*key);
+	*key = NULL;
 
 	/* Create a new key. */
 	if ((*key = RSA_new()) == NULL) {
@@ -175,30 +188,43 @@ crypto_keys_subr_import_RSA_priv(RSA ** key, const uint8_t * buf, size_t buflen)
 	}
 
 	/* Load values. */
-	if (import_BN(&(*key)->n, &buf, &buflen))
-		goto err1;
-	if (import_BN(&(*key)->e, &buf, &buflen))
-		goto err1;
-	if (import_BN(&(*key)->d, &buf, &buflen))
-		goto err1;
-	if (import_BN(&(*key)->p, &buf, &buflen))
-		goto err1;
-	if (import_BN(&(*key)->q, &buf, &buflen))
-		goto err1;
-	if (import_BN(&(*key)->dmp1, &buf, &buflen))
-		goto err1;
-	if (import_BN(&(*key)->dmq1, &buf, &buflen))
-		goto err1;
-	if (import_BN(&(*key)->iqmp, &buf, &buflen))
-		goto err1;
+	if (import_BN(&n, &buf, &buflen))
+		goto err2;
+	if (import_BN(&e, &buf, &buflen))
+		goto err2;
+	if (import_BN(&d, &buf, &buflen))
+		goto err2;
+	if (import_BN(&p, &buf, &buflen))
+		goto err2;
+	if (import_BN(&q, &buf, &buflen))
+		goto err2;
+	if (import_BN(&dmp1, &buf, &buflen))
+		goto err2;
+	if (import_BN(&dmq1, &buf, &buflen))
+		goto err2;
+	if (import_BN(&iqmp, &buf, &buflen))
+		goto err2;
 
 	/* We should have no unprocessed data left. */
 	if (buflen)
+		goto err2;
+
+	/* Load values into the RSA key. */
+	if (crypto_compat_RSA_import(key, n, e, d, p, q, dmp1, dmq1, iqmp))
 		goto err1;
 
 	/* Success! */
 	return (0);
 
+err2:
+	BN_free(n);
+	BN_free(e);
+	BN_clear_free(d);
+	BN_clear_free(p);
+	BN_clear_free(q);
+	BN_clear_free(dmp1);
+	BN_clear_free(dmq1);
+	BN_clear_free(iqmp);
 err1:
 	RSA_free(*key);
 	*key = NULL;
@@ -214,10 +240,15 @@ err0:
 int
 crypto_keys_subr_import_RSA_pub(RSA ** key, const uint8_t * buf, size_t buflen)
 {
+	BIGNUM * n, * e;
+
+	/* This simplifies the error path cleanup. */
+	n = e = NULL;
 
 	/* Free any existing key. */
 	if (*key != NULL)
 		RSA_free(*key);
+	*key = NULL;
 
 	/* Create a new key. */
 	if ((*key = RSA_new()) == NULL) {
@@ -226,18 +257,26 @@ crypto_keys_subr_import_RSA_pub(RSA ** key, const uint8_t * buf, size_t buflen)
 	}
 
 	/* Load values. */
-	if (import_BN(&(*key)->n, &buf, &buflen))
-		goto err1;
-	if (import_BN(&(*key)->e, &buf, &buflen))
-		goto err1;
+	if (import_BN(&n, &buf, &buflen))
+		goto err2;
+	if (import_BN(&e, &buf, &buflen))
+		goto err2;
 
 	/* We should have no unprocessed data left. */
 	if (buflen)
+		goto err2;
+
+	/* Load values into the RSA key. */
+	if (crypto_compat_RSA_import(key, n, e, NULL, NULL, NULL, NULL, NULL,
+	    NULL))
 		goto err1;
 
 	/* Success! */
 	return (0);
 
+err2:
+	BN_free(n);
+	BN_free(e);
 err1:
 	RSA_free(*key);
 	*key = NULL;
@@ -260,6 +299,7 @@ crypto_keys_subr_import_HMAC(struct crypto_hmac_key ** key,
 		free((*key)->key);
 		free(*key);
 	}
+	*key = NULL;
 
 	/* Make sure the buffer is the right length. */
 	if (buflen != 32) {
@@ -298,6 +338,7 @@ err0:
 uint32_t
 crypto_keys_subr_export_RSA_priv(RSA * key, uint8_t * buf, size_t buflen)
 {
+	const BIGNUM * n, * e, * d, * p, * q, * dmp1, * dmq1, * iqmp;
 	uint32_t len = 0;
 
 	if (key == NULL) {
@@ -305,22 +346,27 @@ crypto_keys_subr_export_RSA_priv(RSA * key, uint8_t * buf, size_t buflen)
 		goto err0;
 	}
 
+	/* Get values from the RSA key. */
+	if (crypto_compat_RSA_export(key, &n, &e, &d, &p, &q, &dmp1, &dmq1,
+	    &iqmp))
+		goto err0;
+
 	/* Each large integer gets exported. */
-	if (export_BN(key->n, &buf, &buflen, &len))
+	if (export_BN(n, &buf, &buflen, &len))
 		goto err0;
-	if (export_BN(key->e, &buf, &buflen, &len))
+	if (export_BN(e, &buf, &buflen, &len))
 		goto err0;
-	if (export_BN(key->d, &buf, &buflen, &len))
+	if (export_BN(d, &buf, &buflen, &len))
 		goto err0;
-	if (export_BN(key->p, &buf, &buflen, &len))
+	if (export_BN(p, &buf, &buflen, &len))
 		goto err0;
-	if (export_BN(key->q, &buf, &buflen, &len))
+	if (export_BN(q, &buf, &buflen, &len))
 		goto err0;
-	if (export_BN(key->dmp1, &buf, &buflen, &len))
+	if (export_BN(dmp1, &buf, &buflen, &len))
 		goto err0;
-	if (export_BN(key->dmq1, &buf, &buflen, &len))
+	if (export_BN(dmq1, &buf, &buflen, &len))
 		goto err0;
-	if (export_BN(key->iqmp, &buf, &buflen, &len))
+	if (export_BN(iqmp, &buf, &buflen, &len))
 		goto err0;
 
 	/* Success! */
@@ -339,6 +385,7 @@ err0:
 uint32_t
 crypto_keys_subr_export_RSA_pub(RSA * key, uint8_t * buf, size_t buflen)
 {
+	const BIGNUM * n, * e;
 	uint32_t len = 0;
 
 	if (key == NULL) {
@@ -346,10 +393,15 @@ crypto_keys_subr_export_RSA_pub(RSA * key, uint8_t * buf, size_t buflen)
 		goto err0;
 	}
 
-	/* Each large integer gets exported. */
-	if (export_BN(key->n, &buf, &buflen, &len))
+	/* Get values from the RSA key. */
+	if (crypto_compat_RSA_export(key, &n, &e, NULL, NULL, NULL, NULL, NULL,
+	    NULL))
 		goto err0;
-	if (export_BN(key->e, &buf, &buflen, &len))
+
+	/* Each large integer gets exported. */
+	if (export_BN(n, &buf, &buflen, &len))
+		goto err0;
+	if (export_BN(e, &buf, &buflen, &len))
 		goto err0;
 
 	/* Success! */
@@ -375,6 +427,9 @@ crypto_keys_subr_export_HMAC(struct crypto_hmac_key * key, uint8_t * buf,
 		goto err0;
 	}
 
+	/* Sanity check.  (uint32_t)(-1) is reserved for errors. */
+	assert(key->len < UINT32_MAX - 1);
+
 	if (buf != NULL) {
 		if (buflen < key->len) {
 			warn0("Unexpected end of key buffer");
@@ -385,7 +440,7 @@ crypto_keys_subr_export_HMAC(struct crypto_hmac_key * key, uint8_t * buf,
 	}
 
 	/* Success! */
-	return (key->len);
+	return ((uint32_t)(key->len));
 
 err0:
 	/* Failure! */
@@ -405,15 +460,14 @@ crypto_keys_subr_generate_RSA(RSA ** priv, RSA ** pub)
 		RSA_free(*priv);
 	if (*pub != NULL)
 		RSA_free(*pub);
+	*priv = *pub = NULL;
 
-	*priv = RSA_generate_key(2048, 65537, NULL, NULL);
-	if (*priv == NULL) {
+	if ((*priv = crypto_compat_RSA_generate_key()) == NULL) {
 		warn0("%s", ERR_error_string(ERR_get_error(), NULL));
 		goto err0;
 	}
 
-	*pub = RSAPublicKey_dup(*priv);
-	if (*pub == NULL) {
+	if ((*pub = RSAPublicKey_dup(*priv)) == NULL) {
 		warn0("%s", ERR_error_string(ERR_get_error(), NULL));
 		goto err1;
 	}
