@@ -4,13 +4,17 @@
 #include <sys/stat.h>
 
 #include <assert.h>
+#include <errno.h>
 #include <fcntl.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
+#include "asprintf.h"
 #include "b64encode.h"
 #include "crypto.h"
+#include "insecure_memzero.h"
 #include "readpass.h"
 #include "scryptenc.h"
 #include "sysendian.h"
@@ -56,11 +60,11 @@ static int read_raw(const uint8_t *, size_t,
 static int read_plaintext(const uint8_t *, size_t,
     uint64_t *, const char *, int);
 static int read_encrypted(const uint8_t *, size_t,
-    uint64_t *, const char *, int);
+    uint64_t *, const char *, int, int);
 static int read_base256(const uint8_t *, size_t,
-    uint64_t *, const char *, int);
+    uint64_t *, const char *, int, int);
 static int read_base64(const char *, size_t,
-    uint64_t *, const char *, int);
+    uint64_t *, const char *, int, int);
 
 static int
 read_raw(const uint8_t * keybuf, size_t keylen, uint64_t * machinenum,
@@ -73,10 +77,10 @@ read_raw(const uint8_t * keybuf, size_t keylen, uint64_t * machinenum,
 		goto err0;
 	}
 
-	/* Parse machine number. */
+	/* Parse machine number from the first 8 bytes. */
 	*machinenum = be64dec(keybuf);
 
-	/* Parse keys. */
+	/* Parse keys from the remaining buffer. */
 	if (crypto_keys_import(&keybuf[8], keylen - 8, keys))
 		goto err0;
 
@@ -115,7 +119,7 @@ err0:
 
 static int
 read_encrypted(const uint8_t * keybuf, size_t keylen, uint64_t * machinenum,
-    const char * filename, int keys)
+    const char * filename, int keys, int force)
 {
 	char * pwprompt;
 	char * passwd;
@@ -153,7 +157,8 @@ read_encrypted(const uint8_t * keybuf, size_t keylen, uint64_t * machinenum,
 
 	/* Decrypt the key file. */
 	rc = scryptdec_buf(keybuf, keylen, deckeybuf, &deckeylen,
-	    (const uint8_t *)passwd, strlen(passwd), 0, 0.5, 86400.0, 0);
+	    (const uint8_t *)passwd, strlen(passwd), 0, 0.5, 86400.0, 0,
+	    force);
 	if (rc != 0) {
 		switch (rc) {
 		case 1:
@@ -199,14 +204,14 @@ read_encrypted(const uint8_t * keybuf, size_t keylen, uint64_t * machinenum,
 	 * Don't need this any more.  To simplify error handling, we zero
 	 * this here but free it later.
 	 */
-	memset(passwd, 0, strlen(passwd));
+	insecure_memzero(passwd, strlen(passwd));
 
 	/* Process the decrypted key file. */
 	if (read_plaintext(deckeybuf, deckeylen, machinenum, filename, keys))
 		goto err3;
 
-	/* Clean up. */
-	memset(deckeybuf, 0, deckeylen);
+	/* Clean up.  We only used the first deckeylen values of deckeybuf. */
+	insecure_memzero(deckeybuf, deckeylen);
 	free(deckeybuf);
 	free(passwd);
 	free(pwprompt);
@@ -215,10 +220,14 @@ read_encrypted(const uint8_t * keybuf, size_t keylen, uint64_t * machinenum,
 	return (0);
 
 err3:
-	memset(deckeybuf, 0, keylen);
+	/*
+	 * Depending on the error, we might not know how much data was written
+	 * to deckeybuf, so we play safe by zeroing the entire allocated array.
+	 */
+	insecure_memzero(deckeybuf, keylen);
 	free(deckeybuf);
 err2:
-	memset(passwd, 0, strlen(passwd));
+	insecure_memzero(passwd, strlen(passwd));
 	free(passwd);
 err1:
 	free(pwprompt);
@@ -229,7 +238,7 @@ err0:
 
 static int
 read_base256(const uint8_t * keybuf, size_t keylen, uint64_t * machinenum,
-    const char * filename, int keys)
+    const char * filename, int keys, int force)
 {
 
 	/* Sanity-check size. */
@@ -241,7 +250,7 @@ read_base256(const uint8_t * keybuf, size_t keylen, uint64_t * machinenum,
 	/* Is this encrypted? */
 	if (memcmp(keybuf, "scrypt", 6) == 0)
 		return (read_encrypted(keybuf, keylen,
-		    machinenum, filename, keys));
+		    machinenum, filename, keys, force));
 
 	/* Parse this as a plaintext keyfile. */
 	return (read_plaintext(keybuf, keylen,
@@ -254,7 +263,7 @@ err0:
 
 static int
 read_base64(const char * keybuf, size_t keylen, uint64_t * machinenum,
-    const char * filename, int keys)
+    const char * filename, int keys, int force)
 {
 	uint8_t * decbuf;
 	size_t decbuflen;
@@ -336,11 +345,11 @@ read_base64(const char * keybuf, size_t keylen, uint64_t * machinenum,
 	}
 
 	/* Process the decoded key file. */
-	if (read_base256(decbuf, decpos, machinenum, filename, keys))
+	if (read_base256(decbuf, decpos, machinenum, filename, keys, force))
 		goto err1;
 
 	/* Zero and free memory. */
-	memset(decbuf, 0, decbuflen);
+	insecure_memzero(decbuf, decbuflen);
 	free(decbuf);
 
 	/* Success! */
@@ -349,7 +358,7 @@ read_base64(const char * keybuf, size_t keylen, uint64_t * machinenum,
 err2:
 	warn0("Key file is corrupt on line %zu: %s", lnum, filename);
 err1:
-	memset(decbuf, 0, decbuflen);
+	insecure_memzero(decbuf, decbuflen);
 	free(decbuf);
 err0:
 	/* Failure! */
@@ -357,16 +366,19 @@ err0:
 }
 
 /**
- * keyfile_read(filename, machinenum, keys):
+ * keyfile_read(filename, machinenum, keys, force):
  * Read keys from a tarsnap key file; and return the machine # via the
  * provided pointer.  Ignore any keys not specified in the ${keys} mask.
+ * If ${force} is 1, do not check whether decryption will exceed
+ * the estimated available memory or time.
  */
 int
-keyfile_read(const char * filename, uint64_t * machinenum, int keys)
+keyfile_read(const char * filename, uint64_t * machinenum, int keys, int force)
 {
 	struct stat sb;
 	uint8_t * keybuf;
 	FILE * f;
+	size_t keyfilelen;
 
 	/* Stat the file. */
 	if (stat(filename, &sb)) {
@@ -374,12 +386,15 @@ keyfile_read(const char * filename, uint64_t * machinenum, int keys)
 		goto err0;
 	}
 
-	/* Allocate memory. */
+	/* Validate keyfile size. */
 	if ((sb.st_size == 0) || (sb.st_size > 1000000)) {
 		warn0("Key file has unreasonable size: %s", filename);
 		goto err0;
 	}
-	if ((keybuf = malloc(sb.st_size)) == NULL)
+	keyfilelen = (size_t)(sb.st_size);
+
+	/* Allocate memory. */
+	if ((keybuf = malloc(keyfilelen)) == NULL)
 		goto err0;
 
 	/* Read the file. */
@@ -387,7 +402,7 @@ keyfile_read(const char * filename, uint64_t * machinenum, int keys)
 		warnp("fopen(%s)", filename);
 		goto err1;
 	}
-	if (fread(keybuf, sb.st_size, 1, f) != 1) {
+	if (fread(keybuf, keyfilelen, 1, f) != 1) {
 		warnp("fread(%s)", filename);
 		goto err2;
 	}
@@ -398,7 +413,7 @@ keyfile_read(const char * filename, uint64_t * machinenum, int keys)
 
 	/* If this is a raw key file, process it. */
 	if ((keybuf[0] == 0x00) || (keybuf[0] == 0xff)) {
-		if (read_raw(keybuf, sb.st_size,
+		if (read_raw(keybuf, keyfilelen,
 		    machinenum, filename, keys)) {
 			if (errno)
 				warnp("Error parsing key file: %s", filename);
@@ -406,8 +421,8 @@ keyfile_read(const char * filename, uint64_t * machinenum, int keys)
 		}
 	} else {
 		/* Otherwise, try to base64 decode it. */
-		if (read_base64((const char *)keybuf, sb.st_size,
-		    machinenum, filename, keys)) {
+		if (read_base64((const char *)keybuf, keyfilelen,
+		    machinenum, filename, keys, force)) {
 			if (errno)
 				warnp("Error parsing key file: %s", filename);
 			goto err1;
@@ -415,7 +430,7 @@ keyfile_read(const char * filename, uint64_t * machinenum, int keys)
 	}
 
 	/* Zero and free memory. */
-	memset(keybuf, 0, sb.st_size);
+	insecure_memzero(keybuf, keyfilelen);
 	free(keybuf);
 
 	/* Success! */
@@ -424,7 +439,7 @@ keyfile_read(const char * filename, uint64_t * machinenum, int keys)
 err2:
 	fclose(f);
 err1:
-	memset(keybuf, 0, sb.st_size);
+	insecure_memzero(keybuf, keyfilelen);
 	free(keybuf);
 err0:
 	/* Failure! */
@@ -546,7 +561,7 @@ keyfile_write_file(FILE * f, uint64_t machinenum, int keys,
 	 * Don't need this any more.  To simplify error handling, we zero
 	 * this here but free it later.
 	 */
-	memset(keybuf, 0, keybuflen);
+	insecure_memzero(keybuf, keybuflen);
 
 	/* If we have a passphrase, we want to encrypt. */
 	if (passphrase != NULL) {
@@ -587,13 +602,13 @@ keyfile_write_file(FILE * f, uint64_t machinenum, int keys,
 
 		/* Error out if the encryption failed. */
 		if (rc != 0) {
-			memset(encrbuf, 0, tskeylen + 128);
+			insecure_memzero(encrbuf, tskeylen + 128);
 			free(encrbuf);
 			goto err2;
 		}
 
 		/* Switch key buffers. */
-		memset(tskeybuf, 0, tskeylen);
+		insecure_memzero(tskeybuf, tskeylen);
 		free(tskeybuf);
 		tskeylen = tskeylen + 128;
 		tskeybuf = encrbuf;
@@ -636,7 +651,7 @@ keyfile_write_file(FILE * f, uint64_t machinenum, int keys,
 	}
 
 	/* Zero and free key buffers. */
-	memset(tskeybuf, 0, tskeylen);
+	insecure_memzero(tskeybuf, tskeylen);
 	free(tskeybuf);
 	free(keybuf);
 
@@ -644,10 +659,10 @@ keyfile_write_file(FILE * f, uint64_t machinenum, int keys,
 	return (0);
 
 err2:
-	memset(tskeybuf, 0, tskeylen);
+	insecure_memzero(tskeybuf, tskeylen);
 	free(tskeybuf);
 err1:
-	memset(keybuf, 0, keybuflen);
+	insecure_memzero(keybuf, keybuflen);
 	free(keybuf);
 err0:
 	/* Failure! */
