@@ -1,6 +1,6 @@
 #include "bsdtar_platform.h"
 
-#include <stdlib.h>
+#include <stdint.h>
 #include <string.h>
 
 #include "crypto.h"
