@@ -1,8 +1,11 @@
 #ifndef _NETPROTO_H_
 #define _NETPROTO_H_
 
+#include <stddef.h>
 #include <stdint.h>
 
+#include "warnp.h"
+
 #include "tsnetwork.h"
 
 typedef struct netproto_connection_internal NETPROTO_CONNECTION;
