@@ -6,11 +6,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <unistd.h>
 
 #include "netproto_internal.h"
 #include "network.h"
 #include "sock.h"
 #include "sock_util.h"
+#include "tsnetwork.h"
 #include "warnp.h"
 
 #include "netproto.h"
