@@ -8,10 +8,9 @@
 int dirutil_fsyncdir(const char *);
 
 /**
- * dirutil_needdir(dirname):
- * Make sure that ${dirname} exists (creating it if necessary) and is a
- * directory.
+ * build_dir(dir, diropt):
+ * Makes sure that ${dir} exists, creating it (and any parents) as necessary.
  */
-int dirutil_needdir(const char *);
+int build_dir(const char *, const char *);
 
 #endif /* !_DIRUTIL_H_ */
