@@ -4,11 +4,15 @@
 #include <sys/stat.h>
 
 #include <assert.h>
+#include <errno.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "crypto.h"
+#include "dirutil.h"
 #include "getopt.h"
 #include "imalloc.h"
 #include "keyfile.h"
@@ -58,67 +62,6 @@ usage(void)
 	/* NOTREACHED */
 }
 
-static void
-build_dir(const char *dir, const char *diropt)
-{
-	struct stat sb;
-	char * s;
-	const char * dirseppos;
-
-	/* We need a directory name and the config option. */
-	assert(dir != NULL);
-	assert(diropt != NULL);
-
-	/* Move through *dir and build all parent directories. */
-	for (dirseppos = dir; *dirseppos != '\0'; ) {
-		/* Move to the next '/', or the end of the string. */
-		if ((dirseppos = strchr(dirseppos + 1, '/')) == NULL)
-			dirseppos = dir + strlen(dir);
-
-		/* Generate a string containing the parent directory. */
-		if (asprintf(&s, "%.*s", (int)(dirseppos - dir), dir) == -1) {
-			warnp("Cannot allocate memory");
-			exit(1);
-		}
-
-		/* Does the parent directory exist already? */
-		if (stat(s, &sb) == 0)
-			goto nextdir;
-
-		/* Did something go wrong? */
-		if (errno != ENOENT) {
-			warnp("Cannot stat: %s", s);
-			exit(1);
-		}
-
-		/* Create the directory. */
-		if (mkdir(s, 0700)) {
-			warnp("Cannot create directory: %s", s);
-			exit(1);
-		}
-
-		/* Tell the user what we did. */
-		fprintf(stderr, "Directory %s created for \"%s %s\"\n",
-		    s, diropt, dir);
-
-nextdir:
-		free(s);
-	}
-
-	/* Make sure permissions on the directory are correct. */
-	if (stat(dir, &sb)) {
-		warnp("Cannot stat: %s", dir);
-		exit(1);
-	}
-	if (sb.st_mode & (S_IRWXG | S_IRWXO)) {
-		if (chmod(dir, sb.st_mode & ~(S_IRWXG | S_IRWXO))) {
-			warnp("Cannot sanitize permissions on directory: %s",
-			    dir);
-			exit(1);
-		}
-	}
-}
-
 static void
 lockdirs(const char * odir, const char * ndir, int * odirlock, int * ndirlock)
 {
@@ -492,6 +435,7 @@ main(int argc, char **argv)
 	/* We should have processed all the arguments. */
 	if (argc != 0)
 		usage();
+	(void)argv; /* argv is not used beyond this point. */
 
 	/* Make sure we have the necessary options. */
 	if ((ocachedir == NULL) || (ncachedir == NULL) ||
@@ -499,8 +443,10 @@ main(int argc, char **argv)
 		usage();
 
 	/* Make sure the cache directories exist. */
-	build_dir(ncachedir, "--newcachdir");
-	build_dir(ocachedir, "--oldcachdir");
+	if (build_dir(ncachedir, "--newcachdir"))
+		exit(1);
+	if (build_dir(ocachedir, "--oldcachdir"))
+		exit(1);
 
 	/* Lock the cache directories. */
 	lockdirs(ocachedir, ncachedir, &odirlock, &ndirlock);
@@ -509,7 +455,7 @@ main(int argc, char **argv)
 	if (keyfile_read(nkeyfile, &nmachinenum,
 	    CRYPTO_KEYMASK_SIGN_PRIV | CRYPTO_KEYMASK_ENCR_PUB |
 	    CRYPTO_KEYMASK_AUTH_GET | CRYPTO_KEYMASK_AUTH_PUT |
-            CRYPTO_KEYMASK_HMAC_FILE_WRITE)) {
+            CRYPTO_KEYMASK_HMAC_FILE_WRITE, 0)) {
 		warnp("Cannot read key file: %s", nkeyfile);
 		exit(1);
 	}
@@ -543,7 +489,7 @@ main(int argc, char **argv)
 	if (keyfile_read(okeyfile, &omachinenum,
 	    CRYPTO_KEYMASK_SIGN_PUB | CRYPTO_KEYMASK_ENCR_PRIV |
 	    CRYPTO_KEYMASK_AUTH_GET | CRYPTO_KEYMASK_AUTH_DELETE |
-	    CRYPTO_KEYMASK_HMAC_FILE)) {
+	    CRYPTO_KEYMASK_HMAC_FILE, 0)) {
 		warnp("Cannot read key file: %s", okeyfile);
 		exit(1);
 	}
