@@ -3,9 +3,11 @@
 #include <sys/stat.h>
 
 #include <errno.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include <zlib.h>
 
 #include "ccache_internal.h"
@@ -484,11 +486,11 @@ ccache_entry_writefile(CCACHE_ENTRY * cce, TAPE_W * cookie,
 		 * file is, even if the file has grown since when we called
 		 * lstat on it and the cache is corrupt.
 		 */
-		if ((off_t)(skiplen + chunklen) > cce->size_new)
+		if ((skiplen + (off_t)chunklen) > cce->size_new)
 			break;
 
 		/* Read until we've got the whole chunk. */
-		for (cpos = 0; cpos < chunklen; cpos += lenread) {
+		for (cpos = 0; cpos < chunklen; cpos += (size_t)lenread) {
 			lenread = read(fd, chunkbuf + cpos, chunklen - cpos);
 			if (lenread < 0) {
 				warnp("reading file");
