@@ -69,6 +69,7 @@ static struct option {
 } tar_longopts[] = {
 	{ "absolute-paths",       0, 'P' },
 	{ "aggressive-networking",0, OPTION_AGGRESSIVE_NETWORKING },
+	{ "archive-names",	  1, OPTION_ARCHIVE_NAMES },
 	{ "cachedir",		  1, OPTION_CACHEDIR },
 	{ "cd",                   1, 'C' },
 	{ "check-links",          0, OPTION_CHECK_LINKS },
@@ -89,12 +90,15 @@ static struct option {
 	{ "fast-read",            0, 'q' },
 	{ "file",                 1, 'f' },
 	{ "files-from",           1, 'T' },
+	{ "force-resources",	  0, OPTION_FORCE_RESOURCES },
 	{ "fsck",		  0, OPTION_FSCK },
 	{ "fsck-prune",		  0, OPTION_FSCK_PRUNE },
 	{ "help",                 0, OPTION_HELP },
 	{ "humanize-numbers",	  0, OPTION_HUMANIZE_NUMBERS },
 	{ "include",              1, OPTION_INCLUDE },
+	{ "initialize-cachedir",  0, OPTION_INITIALIZE_CACHEDIR },
 	{ "insane-filesystems",	  0, OPTION_INSANE_FILESYSTEMS },
+	{ "iso-dates",		  0, OPTION_ISO_DATES },
 	{ "insecure",             0, 'P' },
 	{ "interactive",          0, 'w' },
 	{ "keep-going",           0, OPTION_KEEP_GOING },
@@ -124,8 +128,10 @@ static struct option {
 	{ "no-config-include",	  0, OPTION_NO_CONFIG_INCLUDE },
 	{ "no-default-config",	  0, OPTION_NO_DEFAULT_CONFIG },
 	{ "no-disk-pause",	  0, OPTION_NO_DISK_PAUSE },
+	{ "no-force-resources",	  0, OPTION_NO_FORCE_RESOURCES },
 	{ "no-humanize-numbers",  0, OPTION_NO_HUMANIZE_NUMBERS },
 	{ "no-insane-filesystems", 0, OPTION_NO_INSANE_FILESYSTEMS },
+	{ "no-iso-dates",	  0, OPTION_NO_ISO_DATES },
 	{ "no-maxbw",		  0, OPTION_NO_MAXBW },
 	{ "no-maxbw-rate-down",	  0, OPTION_NO_MAXBW_RATE_DOWN },
 	{ "no-maxbw-rate-up",	  0, OPTION_NO_MAXBW_RATE_UP },
@@ -361,7 +367,7 @@ bsdtar_getopt(struct bsdtar *bsdtar)
 			/* Short-circuit if first chars don't match. */
 			if (popt->name[0] != opt_word[0])
 				continue;
-			/* If option is a prefix of name in table, record it.*/
+			/* If option is a prefix of name in table, record it. */
 			if (strncmp(opt_word, popt->name, optlength) == 0) {
 				match2 = match; /* Record up to two matches. */
 				match = popt;
