@@ -1,6 +1,8 @@
 #include "bsdtar_platform.h"
 
 #include <fcntl.h>
+#include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
