@@ -106,7 +106,7 @@ err0:
  */
 STORAGE_D *
 storage_fsck_start(uint64_t machinenum, uint8_t seqnum[32],
-    int readonly, int whichkey)
+    int readonly, uint8_t whichkey)
 {
 	struct storage_delete_internal * S;
 
@@ -167,7 +167,7 @@ storage_delete_file(STORAGE_D * S, char class, const uint8_t name[32])
 		goto err0;
 	C->S = S;
 	C->machinenum = S->machinenum;
-	C->class = class;
+	C->class = (uint8_t)class;
 	memcpy(C->name, name, 32);
 	memcpy(C->nonce, S->nonce, 32);
 	C->done = 0;
