@@ -1,6 +1,7 @@
 #include "bsdtar_platform.h"
 
 #include <assert.h>
+#include <stddef.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -208,7 +209,7 @@ storage_read_cache(STORAGE_R * S, char class, const uint8_t name[32])
 	cache_prune(S);
 
 	/* Is this file already marked as needing to be cached? */
-	classname[0] = class;
+	classname[0] = (uint8_t)class;
 	memcpy(&classname[1], name, 32);
 	if ((CF = rwhashtab_read(S->cache, classname)) != NULL) {
 		/* If we're in the linked list, remove ourselves from it. */
@@ -250,7 +251,7 @@ err0:
 
 /**
  * storage_read_cache_limit(S, size):
- * Set a limit of ${size} bytes on the cache associated with read cookie {S}.
+ * Set a limit of ${size} bytes on the cache associated with read cookie ${S}.
  */
 void
 storage_read_cache_limit(STORAGE_R * S, size_t size)
@@ -273,7 +274,7 @@ storage_read_cache_find(STORAGE_R * S, char class, const uint8_t name[32],
 	*buflen = 0;
 
 	/* Search for a cache entry. */
-	classname[0] = class;
+	classname[0] = (uint8_t)class;
 	memcpy(&classname[1], name, 32);
 	if ((CF = rwhashtab_read(S->cache, classname)) != NULL) {
 		/* Found it! */
@@ -419,14 +420,14 @@ storage_read_file_callback(STORAGE_R * S, uint8_t * buf, size_t buflen,
 	C->cookie = cookie;
 	C->S = S;
 	C->machinenum = S->machinenum;
-	C->class = class;
+	C->class = (uint8_t)class;
 	memcpy(C->name, name, 32);
 
 	/* Do we have a buffer? */
 	if (buf != NULL) {
 		C->buf = buf;
 		C->buflen = buflen;
-		C->size = buflen + STORAGE_FILE_OVERHEAD;
+		C->size = (uint32_t)(buflen + STORAGE_FILE_OVERHEAD);
 	} else {
 		C->buf = NULL;
 		C->buflen = 0;
